{*> Ver: V1.1 *********      History      ***************************\

V1.1            05/06/2001	Released

Legal issues: Copyright (C) 2001 by Boian Mitov
              <mitov@mitov.com>
              <http://www.mitov.com>

              This software is provided 'as-is', without any express or
              implied warranty.  In no event will the author be held liable
              for any  damages arising from the use of this software.

              Permission is granted to anyone to use this software for any
              purpose, including commercial applications, and to alter it
              and redistribute it freely, subject to the following
              restrictions:

              1. The origin of this software must not be misrepresented,
                 you must not claim that you wrote the original software.
                 If you use this software in a product, an acknowledgment
                 in the product documentation would be appreciated but is
                 not required.

              2. Altered source versions must be plainly marked as such, and
                 must not be misrepresented as being the original software.

              3. This notice may not be removed or altered from any source
                 distribution.

\***************************************************************************}
unit BufferListForm;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  StdCtrls, ImgList, Buttons, ComCtrls, ShellAPI;

type
  TBufferListFrm = class(TForm)
    ListView: TListView;
    OKButton: TBitBtn;
    CancelButton: TBitBtn;
    PointsImageList: TImageList;
    SmallImages: TImageList;
    ResultImageList: TImageList;
    procedure FormShow(Sender: TObject);
    procedure ListViewColumnClick(Sender: TObject; Column: TListColumn);
    procedure ListViewCompare(Sender: TObject; Item1, Item2: TListItem;
      Data: Integer; var Compare: Integer);
    procedure ListViewDblClick(Sender: TObject);
  private
    { Private declarations }
    ColumnToSort : Integer;
    Direction    : Boolean;

  public
    { Public declarations }
    constructor Create(AOwner: TComponent ); override;
    
  public
    { Public declarations }
    procedure AddBuffer( BufferName : String );
    procedure Clear();
    function GetSelectedIndex() : Integer;
  end;

implementation

{$R *.DFM}

constructor TBufferListFrm.Create(AOwner: TComponent );
begin
  inherited;
  ResultImageList.Clear();
  ResultImageList.AddImages( PointsImageList );
end;

procedure TBufferListFrm.FormShow(Sender: TObject);
begin
  ListView.AlphaSort();
  ListView.Columns.Items [ ColumnToSort ].ImageIndex := Ord( Direction );
  
  if ( ListView.Selected <> NIL ) then
    ListView.Selected.MakeVisible( False );

  ActiveControl := ListView;
end;

procedure TBufferListFrm.ListViewColumnClick(Sender: TObject;
  Column: TListColumn);
begin
  ListView.Columns.Items [ ColumnToSort ].ImageIndex := -1;

  if ( ColumnToSort = Column.Index ) then
    Direction := not Direction

  else
    begin
    Direction := true;
    ColumnToSort := Column.Index;
    end;

  ListView.AlphaSort();

  ListView.Columns.Items [ ColumnToSort].ImageIndex := Ord(Direction);
end;

procedure TBufferListFrm.AddBuffer( BufferName : String );
var
  Item          : TListItem;
  Ext           : String;
  SHFileInfo    : TSHFileInfo;
  Image         : TIcon; 

begin
  Item := ListView.Items.Add();

  Item.Caption := ExtractFileName(BufferName);
  Ext := ExtractFileExt( BufferName );
  if ( Ext[1] = '.' ) then
    begin
    Delete( Ext, 1, 1 );
    Ext := UpperCase(Ext);
    end;

  Item.SubItems.Add ( Ext );
  Item.SubItems.Add ( ExtractFileDir(BufferName) );

  if( FileExists( BufferName )) then
    begin
    SmallImages.ShareImages := True;

    FillChar(SHFileInfo, SizeOf(SHFileInfo), #0);
    SmallImages.Handle := SHGetFileInfo( PChar(BufferName), 0, SHFileInfo,
            SizeOf(SHFileInfo), SHGFI_SMALLICON or SHGFI_SYSICONINDEX);

    SHGetFileInfo( PChar( BufferName ), 0, SHFileInfo,
            sizeof(SHFileInfo),
            SHGFI_SYSICONINDEX );

    Image := TIcon.Create();
    SmallImages.GetIcon( SHFileInfo.iIcon, Image );
    ResultImageList.AddIcon( Image );

    Image.Free();

    Item.ImageIndex := ResultImageList.Count - 1;
    end

  else
    Item.ImageIndex := 2;

  Item.Data := Pointer( Item.Index );

  if ( ListView.Selected = NIL ) then
    begin
    Item.ImageIndex := 3;
    ListView.Selected := Item;
    ListView.ItemFocused := Item;
//    ListView.Selected := Item;
    end;
    
end;

procedure TBufferListFrm.Clear();
begin
  ListView.Items.BeginUpdate();
  ListView.Items.Clear();
  ListView.Items.EndUpdate();
end;

function  TBufferListFrm.GetSelectedIndex() : Integer;
begin
  Result := 0;
  if ( ListView.Selected <> NIL ) then
    Result := Integer( ListView.Selected.Data );
    
end;

procedure TBufferListFrm.ListViewCompare(Sender: TObject; Item1,
  Item2: TListItem; Data: Integer; var Compare: Integer);
var
  DirMultiply : Integer;
  ix          : Integer;
    
begin
  if( Direction ) then
    DirMultiply := 1
    
  else
    DirMultiply := -1;

  if ( ColumnToSort = 0 ) then
    Compare := DirMultiply * CompareText ( Item1.Caption, Item2.Caption )

  else
    begin
    ix := ColumnToSort - 1;
    Compare := DirMultiply * CompareText ( Item1.SubItems.Strings[ix], Item2.SubItems.Strings[ix] );
    end;

end;

procedure TBufferListFrm.ListViewDblClick(Sender: TObject);
begin
  if ( ListView.Selected <> NIL ) then
    ModalResult := mrOk;
end;

end.
